static const char rcs_id[] = "@(#) $Id: ng_skyfish.c,v 1.00 2008/04/12 nsand Exp $";

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/mbuf.h>
#include <sys/malloc.h>
#include <sys/ctype.h>
#include <sys/errno.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/ucred.h>
#include <sys/sysctl.h>		/* XXX for udp_var.h */
#include <net/bpf.h>
#include <net/ethernet.h>
#include <net/route.h>
#include <sys/proc.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/in_pcb.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>
#include <netinet/tcp_var.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>

#include <netgraph/ng_message.h>
#include <netgraph/ng_parse.h>
#include <netgraph/netgraph.h>


#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>


#include "ng_skyfish.h"
#include "fifobuf.h"
#include "drv.h"


/*
 * FreeBSD version check
 */
#if !defined(__FreeBSD_version) || \
		((__FreeBSD_version < 600000))
#error "Module not supported on this version of FreeBSD."
#endif

#define DEBUG

#ifdef  DEBUG
#define Dbg_print(lvl, args)	do { if (hip->hi.hi_debug & lvl) printf args; } while (0)
#else
#define Dbg_print(lvl, args)
#endif

#ifndef NTOHS
#define NTOHS(a) (a) = ntohs((a))
#endif


/* vars */
//static int count;

static ng_constructor_t	ng_ipacct_constructor;
static ng_shutdown_t	ng_ipacct_shutdown;
static ng_newhook_t	ng_ipacct_newhook;
static ng_rcvdata_t	ng_ipacct_rcvdata;
static ng_disconnect_t	ng_ipacct_disconnect;
static int ng_ipacct_mod_event(module_t mod, int event, void *data);

/* Netgraph node type descriptor */
static struct ng_type ng_ipacct_typestruct = {
	.version = NG_ABI_VERSION,
	.name = NG_IPACCT_NODE_TYPE,
	.mod_event = ng_ipacct_mod_event,
	.constructor = ng_ipacct_constructor,
	.shutdown = ng_ipacct_shutdown,
	.newhook = ng_ipacct_newhook,
	.rcvdata = ng_ipacct_rcvdata,
	.disconnect = ng_ipacct_disconnect,
};

NETGRAPH_INIT(ipacct, &ng_ipacct_typestruct);

/* Information we store for each hook on each node */

struct ipacct_hook {
	hook_p  		hook;
	node_p			node;
//    struct mtx      h_mtx;         /* protects hook data */
//	struct ip_acct_hash  	*active;	/* active database */
//	struct ip_acct_hash	*checked;	/* checkpointed database */
	struct ng_ipacct_hinfo	hi;		/* hook info */
};

typedef struct ipacct_hook *hinfo_p;

/* Inlcude memory allocation primitives */
#include	"ng_ipacct_mem.h"
/* Inlcude hash manipulation primitives */
#include	"ng_ipacct_hash.h"

static int  ip_account_add(struct mbuf *);

static int
ng_ipacct_mod_event(module_t mod, int event, void *data)
{
	switch(event) {
	case MOD_LOAD:
		HASH_MEMINIT();
		
		break;
	case MOD_UNLOAD:
		HASH_MEMFINI();
		break;		
	}

	return (0); 
}

/*
 * Called at node creation
 */

static int
ng_ipacct_constructor(node_p nodep)
{
	return (0);
}

/*
 * Called at hook connection
 */
static int
ng_ipacct_newhook(node_p node, hook_p hook, const char *name)
{

    printf("ng_ipacct_newhook:%s\n",name);
	hinfo_p		hip;
//	int 		error;

	MALLOC(hip, hinfo_p, sizeof(*hip), M_NETGRAPH, M_NOWAIT | M_ZERO);
	if (hip == NULL)
		return (ENOMEM);

	/* allocate space for hash of accounting records */
//	if ((error = ip_hash_init(&hip->active)))
//		return (error);

//	hip->checked = NULL;
	hip->hook = hook;
	hip->node = node;
	/* 
	 *  set DLT to EN10MB (for compatibility reasons), can be changed 
	 *  via ipacctctl <..> dlt DLT
	 */
	hip->hi.hi_dlt = DLT_EN10MB;

	if (!strncmp(name + strlen(name) - 3, "_in", 3))
		hip->hi.hi_flags |= HI_INCOMING_HOOK;
//    mtx_init(&hip->h_mtx, "ng_ipacct hash", NULL, MTX_DEF);
	NG_HOOK_SET_PRIVATE(hook, hip);
	hip->hi.hi_debug = 0;
	return (0);
}

static int
ng_ipacct_rcvdata(hook_p hook, item_p item)
{
	const hinfo_p 	hip = NG_HOOK_PRIVATE(hook);
	int 		error = 0;
	struct mbuf *m;

	rcvdata ++;
	m = NGI_M(item);
	if (hip->hi.hi_dlt == DLT_EN10MB)
	{
		struct ether_header *eh;
		eh = mtod(m, struct ether_header *);
		NTOHS(eh->ether_type);
		if (eh->ether_type == ETHERTYPE_IP) error = ip_account_add(m);
	}
	NG_FREE_ITEM(item);
	return (error);
}

static int
ng_ipacct_shutdown(node_p node)
{
	NG_NODE_UNREF(node);
	return (0);
}

/*
 * Hook disconnection
 *
 * Removal of the last link destroys the node
 */
static int
ng_ipacct_disconnect(hook_p hook)
{
	const hinfo_p hip = NG_HOOK_PRIVATE(hook);

	NG_HOOK_SET_PRIVATE(hook, NULL);
//    mtx_destroy(&hip->h_mtx);
	FREE(hip, M_NETGRAPH);
	if ((NG_NODE_NUMHOOKS(NG_HOOK_NODE(hook)) == 0) &&
		(NG_NODE_IS_VALID(NG_HOOK_NODE(hook)))) {
		ng_rmnode_self(NG_HOOK_NODE(hook));
	}

	return (0);
}

/*
 * Add streams into FIFO buffer in kernel 
 * 
 */
static int
ip_account_add(struct mbuf * m)
{
    rcvdata2++;
    critical_enter();
    u_char *data;
    u_long length = m_length(m,NULL);
    u_long len = m->m_len;
    
    if (len != length) 	{ critical_exit();    return 1;}
    if (len == 0 ) 	{ critical_exit();    return 1;}

    data = mtod(m, u_char*);
    pos_f = pos_w + len + 4;	// future position

    if (pos_w < pos_r && pos_f >= pos_r) {
//	printf("ng_skyfish overflow1!\n");
	critical_exit();	return 1;
    }

    if (pos_w > pos_r)
	if (pos_f >= (BUFFERSIZE - 4))
	    if (len >= pos_r) {
//	    	printf("ng_skyfish overflow2!\n");
	    	critical_exit();	return 1;
	    }
    if (pos_f >= (BUFFERSIZE - 4)) {
	memcpy(fifobuf + pos_w, &len, 4);
	pos_w=0;   
    }

    memcpy(fifobuf + pos_w, &len, 4);
    memcpy(fifobuf + pos_w + 4, data, len);
    pos_w += (len + 4);
    isdata++;
    KNOTE_UNLOCKED(&echo_rsel.si_note, 0);				    
    critical_exit();    
    return 0;
}
